//
//  AppDelegate.m
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/12/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

#import "AppDelegate.h"
#import "NSString+Additions.h"

#define kRegisterKey @"registered"

@interface AppDelegate ()

@end

@implementation AppDelegate


- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    // Override point for customization after application launch.
    
    self.keyStore = [SecKeyWrapper sharedWrapper];
    self.dataStore = [[DataStore alloc] init];
    self.conversationManager = [[ConversationManager alloc] initWithDataStore: self.dataStore keyStore: self.keyStore];
    
    /** Commented out helper to delete all stored state **/
//    [self.keyStore deleteAsymmetricKeys];
//    [self.dataStore deleteContactList];
//    [self.conversationManager deleteAllData];
    
    UIUserNotificationSettings *settings = [UIUserNotificationSettings settingsForTypes: (UIUserNotificationTypeAlert | UIUserNotificationTypeBadge | UIUserNotificationTypeSound) categories:nil];
    [application registerUserNotificationSettings:settings];
    
    return YES;
}

- (void)application:(UIApplication *)application didRegisterUserNotificationSettings:(UIUserNotificationSettings *)notificationSettings {
    //register to receive notifications
    [application registerForRemoteNotifications];
}

- (void)application:(UIApplication *)application handleActionWithIdentifier:(NSString *)identifier forRemoteNotification:(NSDictionary *)userInfo completionHandler:(void(^)())completionHandler
{
    //handle the actions
    if ([identifier isEqualToString:@"declineAction"]){
    }
    else if ([identifier isEqualToString:@"answerAction"]){
    }
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    NSString *deviceTokenString = [NSString hexadecimalFromData: deviceToken];
    
    NSLog(@"device token: %@", deviceTokenString);
    
    self.deviceToken = deviceTokenString;
    [self registerUser];
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error {
    NSLog(@"%@", error);
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo {
    UIApplicationState state = [application applicationState];
    if (state == UIApplicationStateActive) {
        
        [[NSNotificationCenter defaultCenter] postNotificationName: kAppPushNotificationNewMessage object:nil userInfo: userInfo];

    }
}

#pragma mark - Register User 

- (void) registerUser {
    
//    if ([[NSUserDefaults standardUserDefaults] objectForKey: kRegisterKey] != nil) {
//        return;
//    }
    
    if ([self.keyStore getPublicKeyBits] == nil) {
        return;
    }
    
    NSString *fingerprint = [NSString getFingerprintFromPublicKey: [self.keyStore getPublicKeyBits]];
    
    if (!self.deviceToken) {
        return;
    }
    
    API *api = [[API alloc] initWithDataStore: self.dataStore keyStore: self.keyStore];
    
    [api registerUser: fingerprint token: self.deviceToken completionHandler:
     ^(BOOL success, NSError *err) {
         if (err) {
             NSLog(@"%@", err);
             return;
         }
         
         if (success) {
             NSLog(@"Successfully Registerd User with Server");
             [[NSUserDefaults standardUserDefaults] setObject: @"true" forKey: kRegisterKey];
         }
     }];
}

#pragma mark - App Lifecycle

- (void)applicationWillResignActive:(UIApplication *)application {
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
    
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    
    [[NSUbiquitousKeyValueStore defaultStore] synchronize];
    [[NSUserDefaults standardUserDefaults] synchronize];
    [self.conversationManager save];
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    [[NSUbiquitousKeyValueStore defaultStore] synchronize];
    [[NSUserDefaults standardUserDefaults] synchronize];
    [self.conversationManager save];
}

@end
