//
//  DataStore.swift
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/13/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

import Foundation

@objc public class DataStore:NSObject {
    
    let ME_USERNAME_KEY = "me_pgc_z64"
    let USERNAME_LIST_KEY = "friendlist_pgc_z64"
    
    var friendList:NSMutableDictionary
    
    override init() {
        self.friendList = NSMutableDictionary()

        super.init()

        self.initializeFriendList()
        
        NSNotificationCenter.defaultCenter().addObserver(self, selector: "updatedCloudKeyValueStore:", name: NSUbiquitousKeyValueStoreDidChangeExternallyNotification, object: NSUbiquitousKeyValueStore.defaultStore())

    }
    
    private func initializeFriendList() {
        
        let usernameList:NSArray? = NSUbiquitousKeyValueStore.defaultStore().arrayForKey(USERNAME_LIST_KEY)
        
        if (usernameList == nil) {
            return
        }
        
        for userObject in usernameList! {
            
            let user:User = User.deserialize(userObject as NSString)!

            self.friendList.setObject(user, forKey: user.fingerprint)
        }
    }
    
    
    public func deleteUserForFingerprint(fingerprint:String) {
        self.friendList.removeObjectForKey(fingerprint)
        self.save()
    }
    
    public func deleteContactList() {
        self.friendList = NSMutableDictionary()

        NSUbiquitousKeyValueStore.defaultStore().removeObjectForKey(USERNAME_LIST_KEY)
        NSUbiquitousKeyValueStore.defaultStore().synchronize()
    }
    

    public func addNewFriend(user:User) -> Bool {
        
        if (user.username == ME_USERNAME_KEY || user.username == USERNAME_LIST_KEY) {
            return false
        }
        
        if (self.friendList.objectForKey(user.fingerprint) != nil) {
            return false
        }
        
        self.friendList.setObject(user, forKey: user.fingerprint)
        self.save()
        
        return true
    }

    public func setMe(username:NSString) {
        NSUbiquitousKeyValueStore.defaultStore().setString(username, forKey: ME_USERNAME_KEY)
    }
    
    public func myUsername() -> NSString? {
        return NSUbiquitousKeyValueStore.defaultStore().stringForKey(ME_USERNAME_KEY)
    }
    
    public func publicKeyFor(fingerprint: NSString) -> SecKeyRef? {
        let user:User? = self.friendList.objectForKey(fingerprint) as? User
        
        if user == nil {
            return nil
        }
        
        var peerRef:Unmanaged<SecKey> = SecKeyWrapper.sharedWrapper().addPeerPublicKey(user!.fingerprint, keyBits: user!.keyBits()!)
        
        var key =  Unmanaged.fromOpaque(peerRef.toOpaque()).takeRetainedValue() as SecKeyRef
        
        return key
    }
    
    public func save() {
        let updatedUsersList = NSMutableArray()
        
        let allUsers:[AnyObject] = self.friendList.allValues
        
        for user in allUsers {
            if let serializedUser = (user as User).serialize() {
                updatedUsersList.addObject(serializedUser)
            }
        }
        
        NSUbiquitousKeyValueStore.defaultStore().setArray(updatedUsersList, forKey: USERNAME_LIST_KEY)

    }
    
    
    public func updatedCloudKeyValueStore(store:NSUbiquitousKeyValueStore) {
        println("updated cloud yo")

        //store.synchronize()
        self.initializeFriendList()
    }
    
}