//
//  ExchangeViewController.m
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/14/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

#import "ExchangeViewController.h"
#import "UIImage+MDQRCode.h"
#import "UIColor+FlatColors.h"
#import "AppDelegate.h"
#import "AMScanViewController.h"
#import "UserViewController.h"

#define kScanIndex 0
#define kMyKeyIndex 1

@interface ExchangeViewController()<AMScanViewControllerDelegate>{}
@property (nonatomic, strong) AMScanViewController *scanViewController;
@property (nonatomic, strong) UIImageView *keyQrCode;

@property (nonatomic, strong) IBOutlet UISegmentedControl *segmentedControl;

@property (nonatomic, strong) SecKeyWrapper *keyStore;
@property (nonatomic, strong) DataStore *dataStore;

@end
@implementation ExchangeViewController

- (void) viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor flatCloudsColor];
    
    self.scanViewController = [[AMScanViewController alloc] init];
    self.scanViewController.delegate = self;
    self.scanViewController.view.frame = self.view.frame;
    
    self.keyStore = [SecKeyWrapper sharedWrapper];
    self.dataStore = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).dataStore;
    
    self.segmentedControl.tintColor = [UIColor flatTurquoiseColor];
    [self showScanningMode];
    
    
#if TARGET_IPHONE_SIMULATOR
    NSString *filePath = [[NSBundle mainBundle] pathForResource: @"ag.json" ofType:@""];
    NSString *tempPayload = [NSString stringWithContentsOfFile: filePath encoding: NSUTF8StringEncoding error:nil];
    [self showNewUserViewController: tempPayload];
#endif
    
}

- (void) viewWillAppear:(BOOL)animated {
    [super viewWillAppear: animated];
    
    if (self.segmentedControl.selectedSegmentIndex == kScanIndex
        && self.scanViewController.isScanning == NO) {
        [self.scanViewController startScanning];
    }
}

- (void) showScanningMode {
    [self.keyQrCode removeFromSuperview];
    
    [self.view insertSubview: self.scanViewController.view atIndex: 0];
    [self addChildViewController: self.scanViewController];
    [self.scanViewController startScanning];
}

- (void) showMyKeyMode {
    [self.scanViewController stopScanning];
    [self.scanViewController removeFromParentViewController];
    [self.scanViewController.view removeFromSuperview];
    
    if (!self.keyQrCode) {
        NSString *publicKeyString = [[self.keyStore getPublicKeyBits] base64EncodedStringWithOptions: NSDataBase64Encoding64CharacterLineLength];
        NSString *username = [self.dataStore myUsername];
        
        User *me = [[User alloc] initWithUsername: username publicKey: publicKeyString];

        self.keyQrCode = [[UIImageView alloc] initWithImage: [UIImage mdQRCodeForString: [me serialize] size: 150 fillColor: [UIColor blackColor]]];
        self.keyQrCode.center = self.view.center;
    }

    [self.view addSubview: self.keyQrCode];

}

- (IBAction) changeModes: (UISegmentedControl *) sender {
    NSLog(@"%li", (long)sender.selectedSegmentIndex);
    
    if (sender.selectedSegmentIndex == kScanIndex) {
        [self showScanningMode];
    } else if (sender.selectedSegmentIndex == kMyKeyIndex) {
        [self showMyKeyMode];
    }
}

- (void) showNewUserViewController: (NSString *) scannedValue {
    User *scannedUser = [User deserialize: scannedValue];
    
    if (scannedUser == nil) {
        NSLog(@"Invalid User Key");
        return;
    }
    
    [self.segmentedControl setSelectedSegmentIndex: kMyKeyIndex];
    [self showMyKeyMode];
    
    //show add screen
    UINavigationController *userNav = [self.storyboard instantiateViewControllerWithIdentifier: @"UserViewNav"];
    UserViewController *userController = [userNav.viewControllers objectAtIndex: 0];
    userController.user = scannedUser;
    userController.viewMode = ExchangeViewMode;
    [self presentViewController: userNav animated: YES completion: nil];
}

#pragma mark - AMScanViewControllerDelegate

- (void) scanViewController:(AMScanViewController *) aCtler didSuccessfullyScan:(NSString *) scannedValue {
    NSLog(@"scanned value %@", scannedValue);
    
    [self showNewUserViewController: scannedValue];
}

@end
