//
//  ViewController.m
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/12/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

#import "MainViewController.h"
#import "UIColor+FlatColors.h"
#import "UserViewController.h"
#import "NSString+Additions.h"
#import "ConversationsViewController.h"

#define kUserNameTag 0


@interface MainViewController ()<UIAlertViewDelegate>
@property (nonatomic, strong) SecKeyWrapper *keyStore;
@property (nonatomic, strong) DataStore *dataStore;

@property (nonatomic, strong) GenerateView *genView;
@property (nonatomic, strong) NSString *username;
@end

@implementation MainViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.navigationController.navigationBar.tintColor = [UIColor whiteColor];
    self.navigationController.navigationBar.barTintColor = [UIColor flatTurquoiseColor];
    NSDictionary *navbarTitleTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
                                               [UIColor whiteColor],NSForegroundColorAttributeName,nil];
    
    self.navigationController.navigationBar.titleTextAttributes = navbarTitleTextAttributes;
    self.tabBar.tintColor = [UIColor flatTurquoiseColor];
    self.navigationItem.titleView = nil;
    self.title = @"Pretty Good Chat";
    
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithImage: [UIImage imageNamed: @"user"] style: UIBarButtonItemStylePlain target: self action: @selector(showUserDetails)];
    
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCompose target: self action: @selector(composeTapped)];

    
    self.keyStore = [SecKeyWrapper sharedWrapper];
    self.dataStore = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).dataStore;
   
    NSLog(@"Public Key: %@", [self.keyStore getPublicKeyBits]);
    
    if ([self.keyStore getPublicKeyBits] == nil) {
        [self showKeysGenerating];
        return;
    }
}

- (void) composeTapped {
    UINavigationController *contactsNav = [self.storyboard instantiateViewControllerWithIdentifier: @"ContactsNav"];
    
    ContactsViewController *contactsViewController = [contactsNav.viewControllers objectAtIndex:0];
    contactsViewController.delegate = self;
    [self presentViewController: contactsNav animated: YES completion: nil];
}
- (void) showUserDetails {
    UINavigationController *userNav = [self.storyboard instantiateViewControllerWithIdentifier: @"UserViewNav"];
    
    [self presentViewController: userNav animated: YES completion: nil];
}

#pragma mark - Contacts Delegate

- (void)deletedUser {
    [((ConversationsViewController *)[self.viewControllers objectAtIndex: 0]).tableView reloadData];

}
- (void)choseConversation:(Conversation *)conversation {
    [self setSelectedIndex: 0];
    [((ConversationsViewController *)[self.viewControllers objectAtIndex: 0]) openConversation: conversation];
}

#pragma mark - Keys

- (void) showKeysGenerating {
    self.genView = [[[NSBundle mainBundle] loadNibNamed:@"GenerateView" owner:self options:nil] objectAtIndex: 0];
    self.genView.center = CGPointMake(self.view.center.x, -self.view.frame.size.height);
    [self.view addSubview: self.genView];

    [UIView animateWithDuration: 1.0 animations:^{
        self.genView.center = self.view.center;
    }
    completion:^(BOOL finished) {
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
            [self.keyStore generateKeyPair: 2048];
            
            NSLog(@"Public Key: %@", [self.keyStore getPublicKeyBits]);
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.genView generatingFinished];
                [self showUsernameAlertView];
            });
        });
    }];
    
}

- (void) showUsernameAlertView {
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle: @"What's your name?" message: @"this is how your friends will see you " delegate:self cancelButtonTitle: nil otherButtonTitles:@"Done!", nil];
    alertView.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alertView show];

}

#pragma mark - AlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    self.username = [alertView textFieldAtIndex:0].text;
    
    if (self.username.length == 0) {
        [self showUsernameAlertView];
        return;
    }
    
    [self.dataStore setMe: self.username];
    [((AppDelegate *)[[UIApplication sharedApplication] delegate]) registerUser];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
