//
//  ChatMessage.swift
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/17/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

import Foundation

internal let kDateKey        = "date"

internal let SPECIAL_CONTENT_REVOKE = "pgc_z64_action_revoke"

private let kContentKey      = "content"

@objc public class ProtocolMessage:NSObject {

    var date:NSDate
    var content:NSString
    
    var isRevokeMessage:Bool
    
    init(content:String, date:NSDate = NSDate()) {
        self.content = content
        self.date = date
        
        self.isRevokeMessage = (self.content == SPECIAL_CONTENT_REVOKE)
    }
    
    public class func createRevokeMessage() -> ProtocolMessage {
        return ProtocolMessage(content: SPECIAL_CONTENT_REVOKE, date: NSDate())
    }
    
    private class func getDataFormatter() -> NSDateFormatter {
        var dateFormatter:NSDateFormatter = NSDateFormatter()
        dateFormatter.dateFormat = "yyyy-MM-dd'T'HH:mm:ss.SSSZZZ"
        
        return dateFormatter
    }
    
    public func toJson() -> NSString? {
        
        let parameters = NSMutableDictionary()
        parameters.setObject(self.content, forKey: kContentKey)
        parameters.setObject(ProtocolMessage.getDataFormatter().stringFromDate(self.date), forKey: kDateKey)
        
        var err:NSError?
        
        let jsonData = NSJSONSerialization.dataWithJSONObject(parameters, options: NSJSONWritingOptions.PrettyPrinted, error:  &err)
        
        if (err != nil) {
            println(err)
            return nil
        }
        
        return NSString(data: jsonData!, encoding: NSUTF8StringEncoding)
    }
    
    public class func toJsonDictionary(jsonString: NSString) -> NSDictionary? {
        let data = jsonString.dataUsingEncoding(NSUTF8StringEncoding)
        
        var err:NSError?
        
        var jsonResult: NSDictionary = NSJSONSerialization.JSONObjectWithData(data!, options: NSJSONReadingOptions.MutableContainers, error: &err) as NSDictionary
        
        if err != nil {
            println(err)
            return nil
        }
        
        return jsonResult
    }
    
    public func serialize() -> NSString? {
        return self.toJson()
    }
    
    public class func deserialize(object: NSString) -> ProtocolMessage? {

        var jsonObject = ProtocolMessage.toJsonDictionary(object)!
        
        let content  = jsonObject.objectForKey(kContentKey) as NSString?
        let date     = jsonObject.objectForKey(kDateKey) as NSString?

        if (content == nil || date == nil) {
            println("missing parameters")
            return nil
        }
        
        return ProtocolMessage(content: content!, date: ProtocolMessage.getDataFormatter().dateFromString(date!)!)
    }
}
