//
//  User.swift
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/17/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

import Foundation


private let kUsernameKey    = "username"
private let kPublicKey      = "pubkey"

@objc public class User:NSObject {
    
    var username:NSString
    var publicKey:NSString
    var fingerprint:NSString
    var peristantKeyRef:AnyObject?
    
    public init(username:NSString, publicKey:NSString) {
        self.username = username
        self.publicKey = publicKey
        
        let keyBits = NSData(base64EncodedString: publicKey, options: NSDataBase64DecodingOptions.IgnoreUnknownCharacters)
        self.fingerprint = NSString.getFingerprintFromPublicKey(keyBits)
    }
    
    public func keyBits() -> NSData? {
        return NSData(base64EncodedString: self.publicKey, options: NSDataBase64DecodingOptions.IgnoreUnknownCharacters)
    }

    
    public func serialize() -> NSString? {
        
        let parameters = NSMutableDictionary()
        parameters.setObject(self.username, forKey:  kUsernameKey)
        parameters.setObject(self.publicKey, forKey: kPublicKey)
        
        var err:NSError?

        let jsonData = NSJSONSerialization.dataWithJSONObject(parameters, options: NSJSONWritingOptions.PrettyPrinted, error:  &err)
        
        if (err != nil) {
            println(err)
            return nil
        }
        
        return NSString(data: jsonData!, encoding: NSUTF8StringEncoding)
    }
    
    public class func deserialize(object: NSString) -> User? {
        let data = object.dataUsingEncoding(NSUTF8StringEncoding)
        

        if data == nil {
            return nil
        }
        
        var err:NSError?

        var jsonResult: NSDictionary = NSJSONSerialization.JSONObjectWithData(data!, options: NSJSONReadingOptions.MutableContainers, error: &err) as NSDictionary
        
        if err != nil {
            println(err)
            return nil
        }
        

        let username  = jsonResult.objectForKey(kUsernameKey) as NSString?
        let publicKey = jsonResult.objectForKey(kPublicKey) as NSString?
        
        
        if (username == nil || publicKey == nil) {
            println("missing parameters")
            return nil
        }
        
        return User(username: username!, publicKey: publicKey!)
    }

}