//
//  UserViewController.m
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/13/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

#import "UserViewController.h"
#import "UIColor+FlatColors.h"
#import <CommonCrypto/CommonCrypto.h>
#import "NSString+Additions.h"
#import "RTSpinKitView.h"


#define kAddedSuccessTag 1
#define kAddedFailedTag 2

@interface UserViewController ()<UIAlertViewDelegate>

@property (nonatomic, strong) SecKeyWrapper *keyStore;
@property (nonatomic, strong) DataStore *dataStore;

@end

@implementation UserViewController

- (void) viewDidLoad {
    [super viewDidLoad];
    
    self.navigationController.navigationBar.tintColor = [UIColor whiteColor];
    self.navigationController.navigationBar.barTintColor = [UIColor flatTurquoiseColor];
    NSDictionary *navbarTitleTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
                                               [UIColor whiteColor],NSForegroundColorAttributeName,nil];
    
    self.navigationController.navigationBar.titleTextAttributes = navbarTitleTextAttributes;
    
    UIView *leftView = [[UIView alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 15.0f, self.view.bounds.size.height)];
    self.usernameField.leftView = leftView;
    self.usernameField.leftViewMode = UITextFieldViewModeAlways;
    
    //reference app objects
    self.keyStore = [SecKeyWrapper sharedWrapper];
    self.dataStore = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).dataStore;
    
    //view mode
    if (self.viewMode == MeViewMode) {
        self.title = @"My Details";
        self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Done" style: UIBarButtonItemStyleDone target: self action: @selector(closeUserDetails)];
        [self.addRevokeKeysButton setBackgroundColor: [UIColor flatAlizarinColor]];
        [self.addRevokeKeysButton setTitleColor: [[UIColor flatSilverColor] colorWithAlphaComponent: 1.0f] forState:UIControlStateHighlighted];
        [self.addRevokeKeysButton addTarget: self action: @selector(regenerateMyKeys) forControlEvents: UIControlEventTouchUpInside];
        
        self.usernameField.text = [self.dataStore myUsername];
        self.usernameField.enabled = YES;
        self.publicKeyLabel.text = [NSString addSpacesToFingerprint: [[NSString getFingerprintFromPublicKey: [self.keyStore getPublicKeyBits]] uppercaseString]];

    } else if (self.viewMode == ExchangeViewMode) {
        self.title = @"New Contact";
        
        self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Cancel" style: UIBarButtonItemStyleDone target: self action: @selector(closeUserDetails)];

        [self.addRevokeKeysButton setTitle: @"ADD USER" forState: UIControlStateNormal];
        [self.addRevokeKeysButton setBackgroundColor: [UIColor flatEmeraldColor]];
        [self.addRevokeKeysButton setTitleColor: [[UIColor flatSilverColor] colorWithAlphaComponent: 1.0f] forState:UIControlStateHighlighted];
        [self.addRevokeKeysButton addTarget: self action: @selector(addFriend) forControlEvents: UIControlEventTouchUpInside];

        self.usernameField.text = self.user.username;
        self.usernameField.enabled = NO;
        self.publicKeyLabel.text = [NSString addSpacesToFingerprint: [[self.user fingerprint] uppercaseString]];

        
    } else if (self.viewMode == FriendViewMode) {
        self.title = @"User Info";

        self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Done" style: UIBarButtonItemStyleDone target: self action: @selector(closeUserDetails)];

        [self.addRevokeKeysButton setTitle: @"DELETE CONTACT" forState: UIControlStateNormal];
        [self.addRevokeKeysButton setBackgroundColor: [UIColor flatAlizarinColor]];
        [self.addRevokeKeysButton setTitleColor: [[UIColor flatSilverColor] colorWithAlphaComponent: 1.0f] forState:UIControlStateHighlighted];
        [self.addRevokeKeysButton addTarget: self action: @selector(revokeFriend) forControlEvents: UIControlEventTouchUpInside];

        self.usernameField.text = self.user.username;
        self.usernameField.enabled = NO;
        self.publicKeyLabel.text = [NSString addSpacesToFingerprint: [[self.user fingerprint] uppercaseString]];
    }
}


#pragma mark - Revoking and Adding
- (void) regenerateMyKeys {
    [ConfirmationAlert askConfirmationIn: self title: @"Are you sure?" text: @"None of your existing contacts will be able to message you, and all your messages and contacts will be deleted. Only go ahead if you think your private key may have been stolen." accept: @"Generate Keys" cancel: @"Cancel" handler:
     ^(BOOL confirmed) {
         if (confirmed) {
             [self revokeKeysAndRegenerate];
         }
     }];
}

- (void) sendRevokeMessages {
    
    NSString *myFingerprint = [NSString getFingerprintFromPublicKey: [self.keyStore getPublicKeyBits]];
    
    API *api = [[API alloc] initWithDataStore: self.dataStore keyStore: self.keyStore];
    
    for (User *contact in [self.dataStore friendList].allValues) {
        
        UserMessage *revokeMessage = [[UserMessage alloc] initTo: contact.fingerprint from: myFingerprint message:[ProtocolMessage createRevokeMessage] signature: nil digest: nil messageId: nil];
        
        [api sendMessage: revokeMessage completionHandler:
         ^(BOOL success, NSError *err) {
            
             if (err) {
                 NSLog(@"error sending revoke message %@", err);
                 return;
             }
             
             if (success) {
                 NSLog(@"success -> sent revoke message to %@", contact.fingerprint);
             }
        }];
    }
}


- (void) revokeKeysAndRegenerate {
    [self sendRevokeMessages];
    
    [self.keyStore deleteAsymmetricKeys];
    [self.dataStore deleteContactList];
    [((AppDelegate *)[[UIApplication sharedApplication] delegate]).conversationManager deleteAllData];
    
    self.publicKeyLabel.text = @"Generating new...";
    
    [self showKeysGenerating];
}

- (void) showKeysGenerating {
    GenerateView *genView = [[[NSBundle mainBundle] loadNibNamed:@"GenerateView" owner:self options:nil] objectAtIndex: 0];
    genView.center = CGPointMake(self.view.center.x, -self.view.frame.size.height);
    [self.view addSubview: genView];
    
    [UIView animateWithDuration: 1.0 animations:^{
        genView.center = self.view.center;
    }
    completion:^(BOOL finished) {
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
           [self.keyStore generateKeyPair: 2048];
           
           NSLog(@"Public Key: %@", [self.keyStore getPublicKeyBits]);
           
           dispatch_async(dispatch_get_main_queue(), ^{
               [genView generatingFinished];
               
               self.publicKeyLabel.text = [NSString addSpacesToFingerprint: [[NSString getFingerprintFromPublicKey: [self.keyStore getPublicKeyBits]] uppercaseString]];

               [self closeUserDetails];
           });
        });
    }];
    
}


- (void) addFriend {
    
    UIView *dimView = [[UIView alloc] initWithFrame: self.navigationController.view.frame];
    dimView.backgroundColor = [UIColor flatMidnightBlueColor];
    dimView.alpha = 0.5f;
    
    RTSpinKitView *spinKit = [[RTSpinKitView alloc] initWithStyle: RTSpinKitViewStyleChasingDots color: [UIColor flatCloudsColor]];
    spinKit.spinnerSize = 50.0f;
    [spinKit sizeToFit];
    spinKit.center = dimView.center;
    [dimView addSubview: spinKit];
    
    [self.navigationController.view addSubview: dimView];

    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
        BOOL succeeded = [self.dataStore addNewFriend: self.user];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            [dimView removeFromSuperview];
            
            
            if (succeeded) {
                UIAlertView *addedSuccessAlert = [[UIAlertView alloc] initWithTitle: @"Added!" message: @"" delegate: self cancelButtonTitle:nil otherButtonTitles: @"OK", nil];
                addedSuccessAlert.tag = kAddedSuccessTag;
                [addedSuccessAlert show];
                
            } else {
                UIAlertView *addedFailedAlert = [[UIAlertView alloc] initWithTitle: @"Failed to add user" message: @"reason -- todo" delegate: self cancelButtonTitle: @"Cancel" otherButtonTitles: @"Try Again", nil];
                addedFailedAlert.tag = kAddedFailedTag;
                [addedFailedAlert show];
            }

        });
    });


}

- (void) revokeFriend {
    
}


#pragma mark - Close
- (void) closeUserDetails {
    if (self.viewMode == MeViewMode) {
        if (NO ==[self.usernameField.text isEqualToString: [self.dataStore myUsername]]
            && self.usernameField.text.length > 0) {
            
            [self.dataStore setMe: self.usernameField.text];
        }

    }
    [self dismissViewControllerAnimated: YES completion: nil];
}


#pragma mark UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (alertView.tag == kAddedSuccessTag) {
        [self dismissViewControllerAnimated: YES completion: nil];
    } else if (alertView.tag == kAddedFailedTag) {
        if (buttonIndex == 1) {
            [self addFriend];
        }
    }
}
@end
