<?php
require_once 'config.php';
require_once 'MySQL_parent.php';

class MySQL_messages extends MySQL_parent{	
	
	public function add_message($to,$from,$message,$signature,$digest){
		/**
		 * Inserts a new message into the database. Locks the relevant tables.
		 */
		$pre=substr($to,0,1);
		$users_table='users_'.$pre;
		$messages_table='messages_'.$pre;
		$this->open_connection();
		$status=$this->lock_tables($users_table, $messages_table);
		if (!$status){return false;}//failed to lock tables - this should never happen
		
		$query="INSERT INTO $messages_table (to_user, from_user, message, signature, digest, added) values (?,?,?,?,?, now())";
		$stmt=$this->conn->prepare($query);
		$stmt->bind_param('sssss',$to,$from,$message,$signature,$digest); //'i' for integer, 's' for string, 'd' for double, 'b' blob
		$status=$stmt->execute();				
		if ($status==false){return false;}//failed to add message - this should never happen
		
		$query="select username from $users_table where username=? limit 1";
		$stmt=$this->conn->prepare($query);
		$stmt->bind_param('s',$to);
		$stmt->execute();
		$stmt->store_result();
		if ($stmt->num_rows==0){
			$query="insert into $users_table (username, num_messages) values(?,1)";
			$stmt=$this->conn->prepare($query);
			$stmt->bind_param('s',$to);
			$stmt->execute();//for now, I'm assuming that increment will never fail
		}
		else{
			$query="update $users_table set num_messages=num_messages+1 where username=?";
			$stmt=$this->conn->prepare($query);
			$stmt->bind_param('s',$to);
			$stmt->execute();//for now, I'm assuming that increment will never fail
		}
		
		$stmt->close();				
		$this->unlock_tables();
		$this->close_connection();	

		file_get_contents("https://polar-tundra-5280.herokuapp.com/api/send_notification/$to");
		return true;
	}
	
	
	public function get_messages($name,$sender){
		/**
		Retrieves all existing messages for $name.
		If $sender!=null, only retrieves messages sent from $sender.
		 **/
		$pre=substr($name,0,1);
		$users_table='users_'.$pre;
		$messages_table='messages_'.$pre;
		$this->open_connection();
		$status=$this->lock_tables($users_table, $messages_table);
		if (!$status){return false;}//failed to lock tables - this should never happen
		
		$query="select num_messages from $users_table where username=? limit 1";
		$stmt=$this->conn->prepare($query);
		$stmt->bind_param('s',$name); //'i' for integer, 's' for string, 'd' for double, 'b' blob
		$stmt->execute();
		$stmt->bind_result($num_new_msgs);
		$stmt->fetch();
		$stmt->close();
		
		if ($num_new_msgs==0){return false;}//there are no messages for this user
		
		if ($sender!=null){//we want the messages from a specific user
			$query="select * from (select * from $messages_table where to_user=? limit $num_new_msgs) as temp_table where from_user=?";
			$stmt=$this->conn->prepare($query);
			$stmt->bind_param('ss',$name, $sender);
			$stmt->execute();
			$stmt->bind_result($to, $from, $message, $signature, $digest, $date_added, $id);
		}
		else{
			$query="select * from $messages_table where to_user=? limit $num_new_msgs";
			$stmt=$this->conn->prepare($query);
			$stmt->bind_param('s',$name);
			$stmt->execute();
			$stmt->bind_result($to, $from, $message, $signature, $digest, $date_added, $id);
		}
		
		$new_msgs='[';
		
		$msgs_to_delete=array();
		$actual_messages=0;
		
		while ($stmt->fetch()){
			$actual_messages+=1;
			$single_msg="{'to':$to,'from':$from, 'message':$message, 'signature':$signature, 'digest':$digest},";
			$new_msgs.=$single_msg;
			
			$delta=time()-strtotime($date_added)-18000;//delta in seconds between insertion to current time, after adjusting to 5 hour UTC diff in Boston. This means that messages sent in different time zones won't be given the same period of grace (i.e., not exactly 7 days). But this doesn't matter, since 7 days is just a large amount of time meant to prevent attacks
			
			if ($delta>604800){//if more than 7 days old, message should be deleted
				array_push($msgs_to_delete,$id);
			}
		}
		$stmt->close();
		if (sizeof($msgs_to_delete>0)){//if there are IDs to delete, then delete them!
			$delete_all=false;
			if ($actual_messages<$num_new_msgs && $sender==null){//this means that an error has somehow occured, and the amount of messages for the user is less than recorded in the users_? table (i.e., less than the value used in the limit)
				$delete_all=true;
			}
			$this->delete($msgs_to_delete, $messages_table, $users_table, $to, $delete_all);
		}
		
		if (strlen($new_msgs)==1){return false;}//i.e., there is only the opening bracket '[' since there are no new messages
		$new_msgs=substr($new_msgs,0,strlen($new_msgs)-1);
		$new_msgs.=']';
		$this->unlock_tables();
		$this->close_connection();
		return $new_msgs;
	}
	
	private function delete($ids,$messages_table,$users_table, $to, $delete_all){
	 	//Delete from $messages_table and decrement the relevant row in $users_table
		$counter=0;
		foreach($ids as $id){
			$query="delete from $messages_table where id=? limit 1";
			$stmt=$this->conn->prepare($query);
			$stmt->bind_param('s',$id); //'i' for integer, 's' for string, 'd' for double, 'b' blob
			$stmt->execute();
			$counter+=$stmt->affected_rows;
			$stmt->close();
		}				
		if($counter!=0){
			if ($delete_all==true){
				$query="delete from $messages_table where to_user=?";
				$stmt=$this->conn->prepare($query);
				$stmt->bind_param('s',$to); 
				$stmt->execute();
				$stmt->close();
				$query="update $users_table set num_messages=0 where username=? limit 1";
			}
			else{
				$query="update $users_table set num_messages=num_messages-$counter where username=? limit 1";
			}
			$stmt=$this->conn->prepare($query);
			$stmt->bind_param('s',$to);
			$stmt->execute();
			$stmt->close();
		}
		return;
	}
		
	private function lock_tables($users_table, $messages_table){
		//locks the two tables for writing
		$lock="lock tables $users_table write, $messages_table write";
		$res=$this->conn->query($lock);
		if(!$res){return false;}
		return true;
	}
	
	private function unlock_tables(){
		//releases all the locks held by the current session
		$unlock="unlock tables";
		$res=$this->conn->query($unlock);
		if(!$res){return false;}
		return true;
	}
}
?>